/* $Id: main.c 7624 2013-05-06 12:57:34Z chris $ */

#include <assert.h>
#include <string.h>

#include "scsi.h"
#include "subchannel.h"


void usage(const char *self) {
  printf("Usage: %s [-r] [-p] [-t] [-c] [-i] [-e]  device\n", self);
}

int main(int argc, char **argv) {
  const char *self;
  device_handle h;
  int rc;
  size_t s;

  struct timing time = TIMING_INIT();

  int dump_raw = 0;
  int dump_parsed = 0;
  int output_timing = 0;
  int output_catalog = 0;
  int output_isrc = 0;
  int log_errors = 0;

  int had_opts = 0;

  self = *argv;
  argv++;
  argc--;

  while (argc > 0) {
    if ((*argv)[0] != '-')
      break;

    switch ((*argv)[1]) {
    case 'r': dump_raw       = 1; break;
    case 'p': dump_parsed    = 1; break;
    case 't': output_timing  = 1; break;
    case 'c': output_catalog = 1; break;
    case 'i': output_isrc    = 1; break;
    case 'e': log_errors     = 1; break;
    default: usage(self); exit(1); break;
    }

    had_opts = 1;

    argv++;
    argc--;
  }

  if (!had_opts) {
    output_isrc = 1;
  }

  if (argc != 1) {
    usage(self);
    exit(1);
  }

  rc = open_device(argv[0], &h);
  if (rc != 0) {
    errno = rc;
    perror("failed to open device");
    exit(1);
  }

  for (s = 0;; s++) {
    uint8_t buf[96];
    uint8_t qsub_raw[12];
    struct qsub qsub;

    rc = read_sector_raw_subchannels(h, s, 1, buf);
    if (rc != 0) break;

    extract_subchannel(buf, 1, qsub_raw);

    if (dump_raw) {
      int i;
      for (i = 0; i < 12; i++)
        printf("%02x ", qsub_raw[i]);
      printf("\n");
    }


    rc = parse_qsub(qsub_raw, &qsub);

    if (rc == 0) {
      switch (qsub.mode) {
      case QSUB_MODE_TIMING:
        if (dump_parsed)
          printf("%u " TIMING_FMT "\n",
                 (unsigned) qsub.mode,
                 TIMING_ARGS(qsub.u.timing.t));
        time = qsub.u.timing.t;

        if (output_timing)
          printf(TIMING_FMT"\n",
                 TIMING_ARGS(time));
        break;

      case QSUB_MODE_CATALOG:
        if (dump_parsed)
          printf("%u " CATALOG_FMT " %d\n",
                 (unsigned) qsub.mode,
                 CATALOG_ARGS(qsub.u.catalog.catalog),
                 qsub.u.catalog.frame);

        advance_timing(&time, qsub.u.catalog.frame);

        if (output_catalog)
          printf(TIMING_FMT " : " CATALOG_FMT "\n",
                 TIMING_ARGS(time),
                 CATALOG_ARGS(qsub.u.catalog.catalog));
        break;

      case QSUB_MODE_ISRC:
        if (dump_parsed)
          printf("%u " ISRC_FMT " %d\n",
                 (unsigned) qsub.mode,
                 ISRC_ARGS(qsub.u.isrc.isrc),
                 qsub.u.isrc.frame);

        advance_timing(&time, qsub.u.isrc.frame);

        if (output_isrc)
          printf(TIMING_FMT " : " ISRC_FMT "\n",
                 TIMING_ARGS(time),
                 ISRC_ARGS(qsub.u.isrc.isrc));
        break;

      default:
        if (dump_parsed)
          printf("%u unknown mode\n", (unsigned) qsub.mode);

        add_to_timing(&time, 1);
        break;

      }
    } else {
      if (log_errors) {
        if      (rc == E_INVALID_CRC)
          printf("subchannel crc error in sector %llu\n", (unsigned long long)/* no %zu in MSVC*/s);
        else if (rc == E_INVALID_DATA)
          printf("subchannel data error in sector %llu\n", (unsigned long long)/* no %zu in MSVC*/s);
        else
          printf("error in sector %llu: %s\n", (unsigned long long)/* no %zu in MSVC*/s, strerror(rc));
      }

      add_to_timing(&time, 1);
    }
  }

  return 0;
}
