/* $Id: subchannel.h 7624 2013-05-06 12:57:34Z chris $ */

#ifndef SUBCHANNEL_H
#define SUBCHANNEL_H


#include <stdint.h>


#define QSUB_MODE_TIMING 1
#define QSUB_MODE_CATALOG 2
#define QSUB_MODE_ISRC  3

struct msf {
  uint8_t m;
  uint8_t s;
  uint8_t f;
};

#define MSF_INIT()                              \
  {                                             \
    /*.m = */0,                                 \
    /*.s = */0,                                 \
    /*.f = */0,                                 \
  }

#define MSF_FMT "%02d:%02d.%02d"
#define MSF_ARGS(x_)                            \
  ((int) ((x_).m)),                             \
  ((int) ((x_).s)),                             \
  ((int) ((x_).f))

struct timing {
  uint8_t track;
  uint8_t index;
  struct msf rel;
  struct msf abs;
};

#define TIMING_INIT()                           \
  {                                             \
    /*.track = */0,                             \
    /*.index = */0,                             \
    /*.rel = */MSF_INIT(),                      \
    /*.abs = */MSF_INIT(),                      \
  }

#define TIMING_FMT "%2d.%02d " MSF_FMT " " MSF_FMT
#define TIMING_ARGS(x_)                         \
  (x_).track,                                   \
  (x_).index,                                   \
  MSF_ARGS((x_).rel),                           \
  MSF_ARGS((x_).abs)




typedef char catalog[13];
#define CATALOG_FMT "%c%c%c%c%c%c%c%c%c%c%c%c%c"
#define CATALOG_ARGS(x_)                        \
    (x_)[ 0], (x_)[ 1], (x_)[ 2], (x_)[ 3],     \
    (x_)[ 4], (x_)[ 5], (x_)[ 6], (x_)[ 7],     \
    (x_)[ 8], (x_)[ 9], (x_)[10], (x_)[11],     \
    (x_)[12]

typedef char isrc[12];
#define ISRC_FMT "%c%c-%c%c%c-%c%c-%c%c%c%c%c"
#define ISRC_ARGS(x_)                           \
    (x_)[ 0], (x_)[ 1], (x_)[ 2], (x_)[ 3],     \
    (x_)[ 4], (x_)[ 5], (x_)[ 6], (x_)[ 7],     \
    (x_)[ 8], (x_)[ 9], (x_)[10], (x_)[11]

struct qsub {
  uint8_t control:4;
  uint8_t mode:4;
  union {
	struct {
	 struct timing t;
	} timing;
    struct {
      catalog catalog;
      uint8_t frame;
    } catalog;
    struct {
      isrc isrc;
      uint8_t frame;
    } isrc;
  } u;
};

/* tut tut */
#define E_INVALID_CRC 32768
#define E_INVALID_DATA 32769

void add_to_timing(struct timing *timing, int frames);
void advance_timing(struct timing *timing, uint8_t abs_f);
void extract_subchannel(uint8_t *in, unsigned subchannel, uint8_t* out);
int parse_qsub(uint8_t *raw, struct qsub *qsub);


#endif
